# ChangeAreaMeleeRanges.py by stumpy

# Script to change the ranges of melee powers that have the Melee:
# Area Attack ("arc") flag set.
#
# This quick'n'dirty implementation will change the min and max ranges
# of any melee power with that flag set to the specified values, which
# default to minRange=1 and maxRange=5.
#
# None of these functions do anything to powers that aren't melee
# powers with that flag set.  Each prints some info to the console
# about the powers or heroes where powers were changed
#
# Backup files are made for changed HERO files and the powers.dat
# file.
#
# I have designed this to run in game (from the console) to make it
# easier to use.  Because of that, there is no need to have the user
# enter the exact filenames since the datfiles module can figure that
# out on its own.
#
#
# For built-in powers, use
#     ChangeAreaMeleeRanges_BuiltIn(PowerName='', minRange=MIN_DEFAULT, maxRange=MAX_DEFAULT)
# 
# When called with no optional parameters, this changes all qualifying
# powers for that mod to have the new min and max ranges.  Optional
# parameters are
#
#     PowerName: a string (defaults to '') with the exact power name 
#         used in FFEdit.  This use changes only the one power.
#     minRange: an integer (defaults to 1) that specifies the minimum
#         range that the powers are to have.
#     maxRange: an integer (defaults to 5) that specifies the maximum
#         range that the powers are to have.
#
# The function can be called with any or no optional parameters.
#
# Example calls:
#     ChangeAreaMeleeRanges_BuiltIn()
#     ChangeAreaMeleeRanges_BuiltIn(PowerName='minute Patriot Swing')
#     ChangeAreaMeleeRanges_BuiltIn(maxRange=3)
#
#
# For HERO files, use
#     ChangeAreaMeleeRanges_Custom(HeroFile, minRange=MIN_DEFAULT, maxRange=MAX_DEFAULT)
# or
#     ChangeAreaMeleeRanges_AllCustoms(HeroFolder='', minRange=MIN_DEFAULT, maxRange=MAX_DEFAULT).
#
# The first changes the power ranges for the custom hero specified as
# the first parameter to the function.  The HeroFile parameter is NOT
# optional, but it can be either a full pathname or just the name of
# the hero file itself
#
# The second _Custom function hunts through all the HERO files in the
# user's directory and changes any qualifying characters.  If the
# HeroFolder parameter is not given, the functio will try and find the
# standard folder where HERO files are kept and look through all the
# files there.
#
# The optional range parameters for the _Custom functions are the same
# as for the _BuiltIn functions.
#
# Example calls:
#     ChangeAreaMeleeRanges_Custom('kal.hero')
#     ChangeAreaMeleeRanges_Custom('kal.hero', minRange=0)
#     ChangeAreaMeleeRanges_AllCustoms().
#     ChangeAreaMeleeRanges_AllCustoms(maxRange=4).



# These are the defaults that will be used for ranges when the
# functions are called without specified range parameters.
MIN_DEFAULT = 1
MAX_DEFAULT = 5


from datfiles import *
import os
import os.path

def ChangeAreaMeleeRanges_BuiltIn(PowerName='', minRange=MIN_DEFAULT, maxRange=MAX_DEFAULT):
    powers = Campaign_ReadPowers()
    if ( PowerName ) :
        if not PowerName in powers.keys():
            print 'ChangeAreaMeleeRanges_BuiltIn: specified power name %s not in power.dat for this mod.  Quitting.' % (repr(PowerName))
            return
        PowerNames = [PowerName]
    else:
        PowerNames = powers.keys()
    AreaMelees = filter(lambda p,powers=powers: ( powers[p].get('PowerType') == PT_MELEE ) and ( powers[p].get('AttackFlags') & PT_ATTACK_MELEE_AREA ), PowerNames)
    ChangePowers = []
    for pname in AreaMelees:
        if ( powers[pname]['RangeMin'] != minRange ) or ( powers[pname]['RangeMax'] != maxRange ):
            p = powers[pname].copy()
            p['RangeMin'] = minRange
            p['RangeMax'] = maxRange
            ChangePowers.append(p)
    if ChangePowers:
        print 'ChangeAreaMeleeRanges_BuiltIn: changing powers for %s' % map(lambda p: p['PowerName'], ChangePowers)
        Campaign_RewritePowers(FileName='',ChangePowers=ChangePowers)
    else:
        print 'ChangeAreaMeleeRanges_BuiltIn: no area melee attacks found that needed their ranges updated for this mod.'

def ChangeAreaMeleeRanges_Custom(HeroFile, minRange=MIN_DEFAULT, maxRange=MAX_DEFAULT):
    if not os.path.isfile(HeroFile):
        OrigFile = HeroFile
        HeroFile = os.path.join(GetHeroFilePath(),HeroFile)
    if not os.path.isfile(HeroFile):
        print 'ChangeAreaMeleeRanges_Custom: neither file <%s> nor <%s> were found. Quitting.' % (OrigFile,HeroFile,)
        return
    hero = Campaign_ReadHeroFile(HeroFile,ForceRead=1)
    powers = hero.get('powers')
    if not powers:
        print 'ChangeAreaMeleeRanges_Custom: no powers found for %s.' % hero['charName']
        return
    PowerNames = powers.keys()
    AreaMelees = filter(lambda p,powers=powers: ( powers[p].get('PowerType') == PT_MELEE ) and ( powers[p].get('AttackFlags') & PT_ATTACK_MELEE_AREA ), PowerNames)
    ChangePowers = []
    for pname in AreaMelees:
        if ( powers[pname]['RangeMin'] != minRange ) or ( powers[pname]['RangeMax'] != maxRange ):
            ChangePowers.append(pname)
    if ChangePowers:
        skip = 0
        try:
            i = 1
            BackupFileName = '%s.bak' % (HeroFile,)
            while os.path.isfile(BackupFileName):
                i = i + 1   # first numbered .bak will be '.bak2'
                BackupFileName = '%s.bak%d' % (HeroFile,i)
            os.rename(HeroFile,BackupFileName)
        except OSError:
            print 'ChangeAreaMeleeRanges_Custom: Unable to create backup of file <%s>, possible Read-Only problem.  Skipping' % (HeroFile,)
            skip = 1
        except:
            print 'ChangeAreaMeleeRanges_Custom: Unable to create backup of file <%s>.  Skipping' % (HeroFile,)
            skip = 1
        if not skip:
            f = open(BackupFileName,'rb')
            dat = f.read()
            f.close()
            pos = 814
            for i in range(10):
                pname = string.split(dat[pos:],'\x00',1)[0]
                if pname in ChangePowers:
                    print 'ChangeAreaMeleeRanges_Custom: <%s> changing ranges for power %s' % (hero['charName'],repr(pname))
                    dat = dat[:pos+126] + chr(minRange) + chr(maxRange) + dat[pos+128:]
                pos = pos + 184
            f = open(HeroFile,'wb')
            f.write('%s' % dat)
            f.close()
    else:
        print 'ChangeAreaMeleeRanges_Custom: <%s> no area melee attacks found needing changed ranges' % hero['charName']

def ChangeAreaMeleeRanges_AllCustoms(HeroFolder='', minRange=MIN_DEFAULT, maxRange=MAX_DEFAULT):
    if not HeroFolder:
        HeroFolder = GetHeroFilePath()
    else:
        HeroFolder = os.path.abspath(HeroFolder)
    if not os.path.isdir(HeroFolder):
        print 'ChangeAreaMeleeRanges_AllCustoms: Folder <%s> not found. Quitting.' % (HeroFolder,)
        return
    files = os.listdir(HeroFolder)
    HeroFiles = filter(lambda f: string.lower(f[-5:])=='.hero', files)
    HeroFiles = map(lambda f,HeroFolder=HeroFolder: os.path.join(HeroFolder,f), HeroFiles)
    for HeroFile in HeroFiles:
        ChangeAreaMeleeRanges_Custom(HeroFile, minRange=minRange, maxRange=maxRange)
