#! /usr/bin/env python

# Create a python data file with the names of all the saved games in
# the order in which they were last modified.  The ModPath argument,
# if given, is the full pathname of the directory for the mod.  E.g.
# ModPath = 'C:\\Program Files\\Irrational Games\\Freedom
# Force\\Strangers\\'. If it is not given, both functions assume they
# are being run from the console and makes a guess about the correct
# mod path.
#
# By default, it assumes it is running in the first version of the
# game. The version can be specified with a -v or --gameVersion option
# on the command line.  For example:
#     SaveFileTimeInfo.py --gameVersion 1 Data
# or
#     SaveFileTimeInfo.py -v 1 Data
# tells it to save the file time information for the data directory of the
# first game.  For the second version, all the saved games for any mod are
# in the same directory.  That may change a some point (it may be a bug),
# but for now that means that there is never a need to specify the mod
# path for use with the second game.  Simply call
#     SaveFileTimeInfo.py --gameVersion 2
#
# The intent is to create this data file before running DrMike2000's FFX
# Control Centre which scans all these files to make sure the complex
# template attribute is set properly for them. A side-effect of this
# scan is that it resets all the file times, so they whow up in the
# wrong order when the file load dialog is called and a more serious
# consequence (for some) is that it will mess up the datfiles call to
# GetMostRecentSavedGameFile().
#
# By running SaveFileTimeInfo(), the original file order is
# determined and saved in a python-readable file.  Then
# RestoreFileTimeInfo() can be called to reset (update the
# modification time) so that the files sort in their original order.

import sys
import os
import os.path
import getopt

# can be run without arguments within the game
def SaveFileTimeInfo(ModPath=''):
    if not ModPath:
        ModPath = GetCurrentModPath()
    SavedGamePath = os.path.join(ModPath,'SaveGames\\')
    SaveFiles = map(lambda f, SavePath=SavedGamePath: os.path.join(SavePath,f),os.listdir(SavedGamePath))
    SaveFiles.append(os.path.join(SavedGamePath,'quicksave\\game.dat'))
    SaveFiles.append(os.path.join(SavedGamePath,'autosave\\autosave - in base.dat'))
    SaveFiles.append(os.path.join(SavedGamePath,'autosave\\autosave - in mission.dat'))
    SaveFiles = filter(os.path.isfile,SaveFiles)
    SaveFiles = filter(lambda f: f[-len('.dat'):]=='.dat',SaveFiles)
    ##print 'SaveFiles=%s' % SaveFiles
    SaveData = map(lambda f: (os.stat(f).st_atime,os.stat(f).st_mtime,f),SaveFiles)
    SaveData.sort()
    ##for i in SaveData: print '<%s>' % (i,)
    f = open(os.path.join(SavedGamePath,'FileOrderData.txt'),'w')
    f.write('%s' % repr(SaveData))
    f.close()

# When run from within a mod
def GetCurrentModPath():
    for p in sys.path:
        if ( p[:len(os.getcwd())]==os.getcwd() ) and ( p[-len('temp'):]=='temp' ):
            return  p[:-len('temp')]
    return ''

# Returns FF directory when run from within the game. If run from outside the game,
# gameVersion must be specified as either 1 or 2 and it will grab the FF directory
# from the Windows registry.
def GetFFPath(gameVersion=-1):
    if gameVersion == 1:
        import _winreg
        return _winreg.QueryValueEx(_winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE,'SOFTWARE\\Irrational Games\\Freedom Force\\'),'InstallDir')[0]
        return FFdir
    elif gameVersion == 2:
        import _winreg
        return _winreg.QueryValueEx(_winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE,'SOFTWARE\\Irrational Games\\FFVTTR\\'),'InstallDir')[0]
    return ''

# Returns all SaveGames paths for the given FF version.
def GetAllModPaths(gameVersion=-1):
    if ( gameVersion == 1 ) or ( gameVersion == 2 ):
        FFsubdirs = filter(os.path.isfile,map(lambda f, FFdir=GetFFPath(gameVersion=gameVersion):
                                              os.path.join(FFdir,f,'characters.dat'), os.listdir(GetFFPath(gameVersion=gameVersion))))
        FFModDirs = map(os.path.dirname,FFsubdirs)        
        return FFModDirs

def GetFFUserPath():
    return os.path.join(os.path.normpath(os.environ['USERPROFILE']),
                         'Local Settings','Application Data','Irrational Games',
                         'Freedom Force vs the 3rd Reich','User')    

if __name__ == '__main__':
    gameVersion = 1
    optlist, args = getopt.getopt(sys.argv[1:], 'v:',['gameVersion='])
    if optlist and ( optlist[0][0] in ['-v','--gameVersion'] ):
        gameVersion = int(optlist[0][1])
    if len(args) >= 1:   # we were given modpaths on the command line
        for mp in args:
            if os.path.isdir(os.path.join(mp,'SaveGames')): # we got a full path
                print 'SaveFileTimeInfo: working on %s' % (mp,)
                SaveFileTimeInfo(mp)
            elif os.path.isdir(os.path.join(GetFFPath(gameVersion=gameVersion),mp,'SaveGames')): # we got just the mod math subdir
                print 'SaveFileTimeInfo: working on %s' % (mp,)
                SaveFileTimeInfo(os.path.join(GetFFPath(gameVersion=gameVersion),mp))
            else:
                print 'SaveFileTimeInfo: skipping %s' % (mp,)
    else:
        if gameVersion == 1:
            for mp in GetAllModPaths(gameVersion=gameVersion):
                if os.path.isdir(os.path.join(mp,'SaveGames')):
                    print 'SaveFileTimeInfo: working on %s' % (mp,)
                    SaveFileTimeInfo(mp)
                else:
                    print 'SaveFileTimeInfo: skipping %s' % (mp,)
        if gameVersion == 2:
            mp = GetFFUserPath()
            print 'SaveFileTimeInfo: working on %s' % (mp,)
            SaveFileTimeInfo(mp)
