# Some functions to get character information not normally available by reading dat files.
#
# For the most part, the approach is to get the most up-to-date information by looking in
# the most recently save game file the was saved from a mission that comes no later than
# the current one in the campaign.  Failing that, it will try and read characters.dat and
# powers.dat (as needed) to determine the same info.  That information is supplemented by
# the information taken from the DAT files.  Custom heroes not in a Saved game file (as
# in Rumble Room matches) are read from their HERO files.
#
# For documentation, search the Scripting forum on Freedom Reborn for
#     +Documentation +datfiles +chardata
# and subject "datfiles and chardata: documentation...".
#
# (c) stumpy 2005, 2006

#################################
### Utility Functions ###########
#################################

# need this import to cover calls to Object_GetTemplate()
from js import *
from ff import *
from datfiles import *
import os
import os.path
# for opimization only
import time

# Does char have the character attribute attrib?
def Character_HasAttribute(char,attrib,ignoreBought=0):
    try:
        cdata = GetCharacterData(char)
        if ignoreBought:
            return attrib in cdata['characterAttributes']
        else:
            return attrib in cdata['characterAttributes'][:cdata['activeAttributes']]
    except:
        print 'Character_HasAttribute: error checking %s (%s) for %s' % \
              (repr(char),Object_GetTemplate(char),attrib)
        return None

#######################################################################
## Getting Power/Defense information for a character ##################
#######################################################################

# For a given character, returns a list of all of that character's
# melee powers.  If ignoreBought is 0, then (for built-in characters)
# it returns the power even if has not been bought yet.
def Character_GetMeleePowers(char, ignoreBought=0):
##    tstart = time.clock()
    cdata = GetCharacterData(char)
    powers = []
    for pname in cdata['powers'].keys():
        if ignoreBought or ( cdata['powerLevels'][pname] > 0 ):
            power = cdata['powers'][pname]
            if ( 'PowerType' in power.keys() ) and ( power['PowerType'] == PT_MELEE ):
                powers.append(power)
##    print 'Character_GetMeleePowers: (no PD) %s %s %s time=%5.4f' % (char,PT_DAMAGES_BLOCKED[damage_type_blocked],PT_MODES_BLOCKED[attack_modes_blocked],(time.clock()-tstart))
    return powers

# For a given character, returns a list of all of that character's
# ranged powers.  Note that these are strictly ranged powers
# (projectiles, beams, cones, etc.), not direct or special powers that
# have a range.  If ignoreBought is 0, then (for built-in characters)
# it returns the power even if has not been bought yet.
def Character_GetRangedPowers(char, ignoreBought=0):
##    tstart = time.clock()
    cdata = GetCharacterData(char)
    powers = []
    for pname in cdata['powers'].keys():
        if ignoreBought or ( cdata['powerLevels'][pname] > 0 ):
            power = cdata['powers'][pname]
            if ( 'PowerType' in power.keys() ) and ( power['PowerType'] == PT_RANGED ):
                powers.append(power)
##    print 'Character_GetRangedPowers: (no PD) %s %s %s time=%5.4f' % (char,PT_DAMAGES_BLOCKED[damage_type_blocked],PT_MODES_BLOCKED[attack_modes_blocked],(time.clock()-tstart))
    return powers

# For a given character, returns a list of all of that character's
# area powers.  If ignoreBought is 0, then (for built-in characters)
# it returns the power even if has not been bought yet.
def Character_GetAreaPowers(char, ignoreBought=0):
##    tstart = time.clock()
    cdata = GetCharacterData(char)
    powers = []
    for pname in cdata['powers'].keys():
        if ignoreBought or ( cdata['powerLevels'][pname] > 0 ):
            power = cdata['powers'][pname]
            if ( 'PowerType' in power.keys() ) and ( power['PowerType'] == PT_AREA ):
                powers.append(power)
##    print 'Character_GetAreaPowers: (no PD) %s %s %s time=%5.4f' % (char,PT_DAMAGES_BLOCKED[damage_type_blocked],PT_MODES_BLOCKED[attack_modes_blocked],(time.clock()-tstart))
    return powers

# For a given character, returns a list of all of that character's
# direct powers.  If ignoreBought is 0, then (for built-in characters)
# it returns the power even if has not been bought yet.
def Character_GetDirectPowers(char, ignoreBought=0):
##    tstart = time.clock()
    cdata = GetCharacterData(char)
    powers = []
    for pname in cdata['powers'].keys():
        if ignoreBought or ( cdata['powerLevels'][pname] > 0 ):
            power = cdata['powers'][pname]
            if ( 'PowerType' in power.keys() ) and ( power['PowerType'] == PT_DIRECT ):
                powers.append(power)
##    print 'Character_GetDirectPowers: (no PD) %s %s %s time=%5.4f' % (char,PT_DAMAGES_BLOCKED[damage_type_blocked],PT_MODES_BLOCKED[attack_modes_blocked],(time.clock()-tstart))
    return powers

# For a given character, returns a list of all of that character's
# special powers.  If ignoreBought is 0, then (for built-in characters)
# it returns the power even if has not been bought yet.
def Character_GetSpecialPowers(char, ignoreBought=0):
##    tstart = time.clock()
    cdata = GetCharacterData(char)
    powers = []
    for pname in cdata['powers'].keys():
        if ignoreBought or ( cdata['powerLevels'][pname] > 0 ):
            power = cdata['powers'][pname]
            if ( 'PowerType' in power.keys() ) and ( power['PowerType'] == PT_SPECIAL ):
                powers.append(power)
##    print 'Character_GetSpecialPowers: (no PD) %s %s %s time=%5.4f' % (char,PT_DAMAGES_BLOCKED[damage_type_blocked],PT_MODES_BLOCKED[attack_modes_blocked],(time.clock()-tstart))
    return powers

# For a given character, returns a list of all of that character's
# powers with the given damage type.  If ignoreBought is 0, then (for
# built-in characters) it returns the power even if has not been
# bought yet.  This ignores special damage type powers; if the power
# is really an alteration power, it won't show up as a crushing power,
# even if that is also true (and irrelevant).
def Character_GetDamageTypePowers(char, DamageType, ignoreBought=0):
##    tstart = time.clock()
    cdata = GetCharacterData(char)
    powers = []
    for pname in cdata['powers'].keys():
        if ignoreBought or ( cdata['powerLevels'][pname] > 0 ):
            power = cdata['powers'][pname]
            if ( 'DamageType' in power.keys() ) and ( power['DamageType'] == DamageType ) and \
               ( power['SpecialType'] == PT_SPECIAL_NONE ):
                powers.append(power)
##    print 'Character_GetDamageTypePowers: (no PD) %s %s %s time=%5.4f' % (char,PT_DAMAGES_BLOCKED[damage_type_blocked],PT_MODES_BLOCKED[attack_modes_blocked],(time.clock()-tstart))
    return powers

# For a given character, returns a list of all of that character's
# powers with the given damage type.  If ignoreBought is 0, then (for
# built-in characters) it returns the power even if has not been
# bought yet.  This ignores special damage type powers; if the power
# is really an alteration power, it won't show up as a crushing power,
# even if that is also true (and irrelevant).
def Character_GetSpecialDamageTypePowers(char, SpecialType, ignoreBought=0):
##    tstart = time.clock()
    cdata = GetCharacterData(char)
    powers = []
    for pname in cdata['powers'].keys():
        if ignoreBought or ( cdata['powerLevels'][pname] > 0 ):
            power = cdata['powers'][pname]
            if ( 'SpecialType' in power.keys() ) and ( power['SpecialType'] == SpecialType ):
                powers.append(power)
##    print 'Character_GetSpecialDamageTypePowers: (no PD) %s %s %s time=%5.4f' % (char,PT_DAMAGES_BLOCKED[damage_type_blocked],PT_MODES_BLOCKED[attack_modes_blocked],(time.clock()-tstart))
    return powers


# This is the general power-finding function.  For a given character,
# returns the list of all the character's powers that match the given
# two-key filter criteria.  The first two items are one of the power
# keys paired with the value it must have, and so on.  For example, if
# I want to find all of the character's ranged radiation attacks, I
# might call
#   Character_GetPower('hero_1', 'PowerType', datfiles.PT_RANGED, 'DamageType', datfiles.PT_DAMAGE_RADIATION)
# If ignoreBought is 0, then (for built-in characters) it returns
# the power even if has not been bought yet.
# Fixed check for multiple value keys, as per Epimethee's suggestion.
def Character_GetPower(char, key1, val1, key2, val2, ignoreBought=0):
##    tstart = time.clock()
    cdata = GetCharacterData(char)
    plist = []
    MultiValKeys = ('AttackFlags', 'AttackModesBlocked', 'DamageTypesBlocked', 'DefenceFlags')
    for pname in cdata['powers'].keys():
        if ignoreBought or ( cdata['powerLevels'][pname] > 0 ):
            power = cdata['powers'][pname]
            if ( key1 in power.keys() ) and ( key2 in power.keys() ) and \
               ( ( power[key1] == val1 ) or ( ( key1 in MultiValKeys ) and ( power[key1] & val1 ) ) ) \
               and \
               ( ( power[key2] == val2 ) or ( ( key2 in MultiValKeys ) and ( power[key2] & val2 ) ) ):
                plist.append(power)
##    print 'Character_GetPower: (no PD) %s %s %s time=%5.4f' % (char,PT_DAMAGES_BLOCKED[damage_type_blocked],PT_MODES_BLOCKED[attack_modes_blocked],(time.clock()-tstart))
    return plist

# For a given character, returns the fraction of successful blocking
# for a passive defence against a given damage type (one of the
# PT_DAMAGE_BLOCKED_* constants) and a given attack mode (one of the
# PT_*_BLOCKED constants).  If ignoreBought is 0, then (for built-in
# characters) it returns the power even if has not been bought yet.
PDefenceSuccessIncrements = [0.00,0.02,0.05,0.07,0.10]
def Character_HasPassiveDefence(char, damage_type_blocked, attack_mode_blocked, ignoreBought=0):
##    tstart = time.clock()
    cdata = GetCharacterData(char)
    for pname in cdata['powers'].keys():
        if ignoreBought or ( cdata['powerLevels'][pname] > 0 ):
            power = cdata['powers'][pname]
            if power['PowerType'] == PT_PASSIVE_DEFENCE:
                if ( power['DamageTypesBlocked'] & damage_type_blocked ) and \
                   ( power['AttackModesBlocked'] & attack_mode_blocked ) :
                    block_pct = PT_BLOCK_SUCCESS_MIN_PCTS[power['Success']] +\
                                PDefenceSuccessIncrements[cdata['powerLevels'][pname]-1] +\
                                Character_HasAttribute(char,'untouchable',ignoreBought)*0.20
                    ##print 'Character_HasPassiveDefence: (custom) %s %s %s time=%5.4f' % (char,PT_DAMAGES_BLOCKED[damage_type_blocked],PT_MODES_BLOCKED[attack_modes_blocked],(time.clock()-tstart))
                    return block_pct
##    print 'Character_HasPassiveDefence: (no PD) %s %s %s time=%5.4f' % (char,PT_DAMAGES_BLOCKED[damage_type_blocked],PT_MODES_BLOCKED[attack_modes_blocked],(time.clock()-tstart))
    return 0

# Split the active defence check into non-remote and remote versions,
# as per Epimethee's suggestion

# For a given character char, returns the (non-zero) PT_BLOCK_TYPE if
# char has an non-remote active defence against the given damage type
# (one of the PT_DAMAGE_BLOCKED_* constants) and the given attack mode
# (one of the PT_*_BLOCKED constants).  If char doesn't have a
# matching defence, then 0 is returned.  This function does not tell
# whether the defence is activated currently.  If ignoreBought is 0,
# then (for built-in characters) it returns the power even if has not
# been bought yet.
def Character_HasActiveDefence(char, damage_type_blocked, attack_mode_blocked, ignoreBought=0):
##    tstart = time.clock()
    cdata = GetCharacterData(char)
    for pname in cdata['powers'].keys():
        if ignoreBought or ( cdata['powerLevels'][pname] > 0 ):
            power = cdata['powers'][pname]
            if power['PowerType'] == PT_ACTIVE_DEFENCE and \
               not (power['DefenceFlags'] & PT_DEFENCE_FLAG_REMOTE):
                if ( power['DamageTypesBlocked'] & damage_type_blocked ) and \
                   ( power['AttackModesBlocked'] & attack_mode_blocked ) :
                    ##print 'Character_HasActiveDefence: (custom) %s %s %s time=%5.4f' % (char,PT_DAMAGES_BLOCKED[damage_type_blocked],PT_MODES_BLOCKED[attack_modes_blocked],(time.clock()-tstart))
                    return power['BlockType']
##    print 'Character_HasActiveDefence: (no PD) %s %s %s time=%5.4f' % (char,PT_DAMAGES_BLOCKED[damage_type_blocked],PT_MODES_BLOCKED[attack_modes_blocked],(time.clock()-tstart))
    return 0

# For a given character char, returns the (non-zero) PT_BLOCK_TYPE if
# char has a remote active defence against the given damage type
# (one of the PT_DAMAGE_BLOCKED_* constants) and the given attack mode
# (one of the PT_*_BLOCKED constants).  If char doesn't have a
# matching defence, then 0 is returned.  This function does not tell
# whether the defence is activated currently.  If ignoreBought is 0,
# then (for built-in characters) it returns the power even if has not
# been bought yet.
def Character_HasRemoteDefence(char, damage_type_blocked, attack_mode_blocked, ignoreBought=0):
##    tstart = time.clock()
    cdata = GetCharacterData(char)
    for pname in cdata['powers'].keys():
        if ignoreBought or ( cdata['powerLevels'][pname] > 0 ):
            power = cdata['powers'][pname]
            if power['PowerType'] == PT_ACTIVE_DEFENCE and \
               (power['DefenceFlags'] & PT_DEFENCE_FLAG_REMOTE):
                if ( power['DamageTypesBlocked'] & damage_type_blocked ) and \
                   ( power['AttackModesBlocked'] & attack_mode_blocked ) :
                    ##print 'Character_HasActiveDefence: (custom) %s %s %s time=%5.4f' % (char,PT_DAMAGES_BLOCKED[damage_type_blocked],PT_MODES_BLOCKED[attack_modes_blocked],(time.clock()-tstart))
                    return power['BlockType']
##    print 'Character_HasActiveDefence: (no PD) %s %s %s time=%5.4f' % (char,PT_DAMAGES_BLOCKED[damage_type_blocked],PT_MODES_BLOCKED[attack_modes_blocked],(time.clock()-tstart))
    return 0

#######################################################################
## The basic interface into the lower-level data reading functions ####
#######################################################################

# Return the data read from a hero file given a custom character's
# in-game object name.  Note that this guessing/matching scheme is
# needed because the template used in the game for custom characters is
# not stored anywhere in the hero file.  Because of this, running
# DrMike2000's FFX Control Centre, which adds a unique value for the
# complex template attribute for each character is a very useful thing,
# even if one is not otherwise using FFX.
#
# HOWEVER, the FFX Control Centre, in updateing the complex attribute,
# reads in each of the saved game files in alphabetical order and
# saves them, messing up the saved game file's original save date and
# foiling functions like GetMostRecentSavedGameFile().  Because of
# that, one should import SaveFileTimeInfo and run SaveFileTimeInfo()
# from the console before running Control Centre, then run Control
# Centre, and then run RestoreFileTimeInfo() from the console.  This
# will re-assert the pre-Control Centre file modifications times to
# each file.
def GetCharacterData(object):
    global _CharData        # The complex-indexed dict of character data
    global _HeroIndexData   # The complex-indexed dict of character data locations
    global _SavedData       # The template-indexed dict of character data

    complex = Object_GetAttr(object,'complex')

    # is it cached?
    if '_CharData' in globals().keys():
        if complex in _CharData.keys():
            return _CharData[complex]
    else:
        _CharData = {}

    if not ( Object_GetClass(object) & OC_CHARACTER ):
        print 'GetCharacterData: called on non-character %s, template=%s' % (object,Object_GetTemplate(object))
        return { 'charName' : Object_GetTemplate(object) }
        
    if isCampaign():    # inside a campaign
        template = Object_GetTemplate(object)
        if '_SavedData' not in globals().keys():
            _SavedData = Campaign_ReadCharactersFromSavedGame()
        if template in _SavedData.keys():
            _CharData[complex] = _SavedData[template]
            return _CharData[complex]

    # If we're still here, this is 1) a danger room, 2) a new hero, or 3) there are no saved games.

    # Let's see where he's at
    if '_HeroIndexData' not in globals().keys():
        _HeroIndexData = ReadHeroIndexData()
    # check if complex is in _HeroIndexData and generate file now if it's not
    if complex not in _HeroIndexData.keys():
        Campaign_MakeHeroIndex(ForceRead=1)
        _HeroIndexData = ReadHeroIndexData()
        if complex not in _HeroIndexData.keys():
            print 'GetCharacterData: unable to determine which character has \'complex\'=%d.  Re-run FFX Control Centre' % complex
            return {}
    hname = _HeroIndexData[complex]
    
    if hname[-5:] != '.hero':   # This char is a built-in
        _CharData[complex] = Campaign_ReadCharacterData()[hname]
    else:                       # This is a custom
        _CharData[complex] = Campaign_ReadHeroFile(hname)

    return _CharData[complex]


def isCampaign():
    return 'OnGameTypeCheck' not in dir(ff)

# Run only from within the game.
def ReadHeroIndexData():
    filename = os.path.join(GetModPath(),'HeroIndexData.txt')
    try:
        f = open(filename,'r')
        s = f.read()
        f.close()
        return eval(s)
    except:
        print 'ReadHeroIndexData: trouble reading or evaluating data from <%s>' % filename
        return {}

def Mission_AttrExists(attrName):
    try:
        dummy = Mission_GetAttr(attrName)
        return 1
    except:
        return 0

#######################################################################
## Determining animation end times for a character ####################
#######################################################################

# Given the character object and power name, return the animation#
# 'end' time for that power.
def Character_GetPowerAnimationTime(object,PowerName,ForceRead=0, verbose=0):
    chardata = GetCharacterData(object)
    AnimName = chardata['powers'][PowerName]['animation']
    KFPath = GetKeyframeFile(object)
    animations = Keyframe_ReadAnimationTimes(KFPath,ForceRead=ForceRead,verbose=verbose)
    if AnimName in animations.keys():
        return animations[AnimName][-1][1]
    else:
        print 'Character_GetPowerAnimationTime: %s not found in animations for %s (%s)' % \
              (AnimName,object,Object_GetTemplate(object))
        return -1

#######################################################################
## Determining keyframe file locations for a character ################
#######################################################################

# Return the location of a character's keyframe file.  The argument
# given is the object name (not the template).  Must be run from
# within the game. 

# *** Maybe this should *not* unzip the art.ff files for the given
# character.  Instead, just check if they are in the art.ff file and
# then return that file's location.  Then, in the ReadKF func, if the
# file is art.ff, unzip the KF to memory there.  Advantage: no
# creation of extra files and no decisions about where to unzip the
# files.  Disadvantage: need to figure some way to uniquely name the
# file this func returns so that two KFs in the art.ff file don't get
# cached as the same KF in the ReadKF func.
def GetKeyframeFile(object):
    chardata = GetCharacterData(object)
    charNIF = chardata['NIF']
    if ffver < 2:
        kfbasenames = ('keyframes.kf','character.kf')
    else:
        kfbasenames = ('character.kf','keyframes.kf','keyframes.txt')
    KFpath = None
    charKFs = map(lambda bn, charNIF=charNIF: os.path.join('art',os.path.dirname(charNIF),bn),kfbasenames)
    for charKF in charKFs:
        # try the windows user temp art path first
        if os.path.isfile(os.path.join(GetWinTempPath(),'custom',charKF)):
            KFpath =  os.path.join(GetWinTempPath(),'custom',charKF)
            break
        # then the mod art path
        elif os.path.isfile(os.path.join(GetModPath(),charKF)):
            KFpath =  os.path.join(GetModPath(),charKF)
            break
        # then the game art path
        elif os.path.isfile(os.path.join(GetFFPath(),'Data',charKF)):
            KFpath =  os.path.join(GetFFPath(),'Data',charKF)
            break
        elif os.path.isfile(os.path.join(GetFFPath(),'custom',charKF)):
            KFpath =  os.path.join(GetFFPath(),'custom',charKF)
            break
    if KFpath:
        if os.path.basename(charKF) != 'keyframes.txt':   # actual KF file
            return KFpath
        else:                           # "soft-link"
            f = open(KFpath,'r')
            softpath = string.strip(f.readline())
            f.close()
            if os.path.isfile(os.path.join(GetFFPath(),softpath)):
                return os.path.join(GetFFPath(),softpath)
            else:   # maybe it's zipped...
                if string.lower(softpath[:5]) == 'data\\':  # strip of mod path (it's not in art.ff list)
                    softpath = softpath[5:]
                hardKFpath = UnzipMesh([softpath],os.path.join(GetFFPath(),'Data\\art.ff'))
                if not hardKFpath:
                    print 'GetKeyframeFile: soft-link <%s> to <%s> not found as file or in art.ff' % \
                          (KFpath, softpath)
                return hardKFpath
    # add extra try layer here to attempt unzip from Data\art.ff
    return UnzipMesh(charKFs[:2],os.path.join(GetFFPath(),'Data\\art.ff'))

# Check the given ZipPath archive for the keyframe files in the
# KFtests list.  If a match is found, unzip the associated mesh and
# return the math name to the KF file.
def UnzipMesh(KFtests, ZipPath):
    if os.path.isfile(ZipPath):
        try:
            import zipfile
            zf = zipfile.ZipFile(ZipPath,'r')
            zipnamelist = zf.namelist()
            zipnormlist = map(os.path.normpath,map(string.lower,zipnamelist))
            ##print 'zipnormlist=%s' % zipnormlist  # WARNING: very long!
            KFpath = None
            for KF in KFtests:
                KFlower = string.lower(KF)
                ##print 'KFlower = %s' % KFlower
                if KFlower in zipnormlist:
                    KFpath = os.path.join(GetFFPath(),'Data',KF)
                    break
            ##print 'UnzipMesh: KFpath = <%s>' % KFpath
            if KFpath:
                # Get all of the zipped files for this mission
                KFdirlower = os.path.dirname(KFlower)                     # in Win format
                KFModDir = os.path.join(GetFFPath(),'Data')   # in Win format
                ##print 'KFModDir = %s' % KFModDir
                KFzips = filter(lambda f, KFDL=KFdirlower: f[:len(KFDL)]==KFDL, zipnormlist)     # in Win format, lowercase
                ##print 'KFzips = %s' % KFzips
                for f in KFzips:
                    ##print 'UnzipMesh: unzipping <%s>' % f
                    unzippedname = os.path.join(KFModDir,f) # Win Format
                    ##print 'unzippedname = %s' % unzippedname
                    if not os.path.isfile(unzippedname):
                        fulldirname = os.path.dirname(unzippedname)
                        if not os.path.isdir(fulldirname):
                            os.makedirs(fulldirname)
                        zipfname = zipnamelist[zipnormlist.index(f)]            # unix format
                        dat = zf.read(zipfname)
                        ##print '    GetKeyframeFile: unzipping  %s' % (unzippedname,)
                        f = open(unzippedname,'wb')
                        f.write(dat)
                        f.close()
                zf.close()
            else:
                print 'GetFromZip: no file in %s found in archive <%s>' % (KFtests,ZipPath,)
            return KFpath
        except ImportError:
            print 'GetFromZip: unable to import zipfile module'
            pass
    else:
        print 'GetFromZip: no zip archive %s found' % (ZipPath,)
    return None

#######################################################################
## A test power as an example #########################################
#######################################################################

def AdaptrixVisionChange(target,char):
    powername = 'microwave Microwave Beam' # power name in powers.dat
    power = Campaign_ReadPowers()[powername]
    mat = Object_GetAttr(target,'material')
    if ( mat == 4 ):                    # fire
        power['DamageType'] = PT_DAMAGE_COLD
    elif ( mat == 9 ) or ( mat == 7 ):  # frozen or wood
        power['DamageType'] = PT_DAMAGE_FIRE
    elif ( mat == 1 ):                  # metal
        power['DamageType'] = PT_DAMAGE_ELECTRICAL
    else:
        power['DamageType'] = PT_DAMAGE_ENERGY
    Campaign_RewritePowers('',[power])
    loadPowers()
    
def AdaptivePower(target,powername=''):
    if not powername:
        powername = 'microwave Microwave Beam' # power name in powers.dat
    power = Campaign_ReadPowers()[powername]
    mat = Object_GetAttr(target,'material')
    if ( mat == 4 ):                    # fire
        power['DamageType'] = PT_DAMAGE_COLD
    elif ( mat == 9 ) or ( mat == 7 ):  # frozen or wood
        power['DamageType'] = PT_DAMAGE_FIRE
    elif ( mat == 1 ):                  # metal
        power['DamageType'] = PT_DAMAGE_ELECTRICAL
    else:
        power['DamageType'] = PT_DAMAGE_ENERGY
    Campaign_RewritePowers('',[power])
    loadPowers()

# Modified from the 16b_homecoming2/mission.py script of DrMike2000's Strangers mod.

# Returns the chance of being affected by a mental attack.  A lower
# number means the character is more resistant to mental damage.  This
# is somewhat arbitrary.  It assumes a basechance of being affected
# and then modifies that for passive mental defenses and mental
# character attributes.
def GetMentalDamageFraction(char,basechance=50.0):
    # speed things up with a little cacheing
    global MentalDamageData
    if 'MentalDamageData' in globals().keys():
        if char in MentalDamageData.keys():
            return MentalDamageData[char]
    else:
        MentalDamageData = {}
    MentalPassiveD = Character_HasPassiveDefence(char,PT_DAMAGE_BLOCKED_MENTAL,PT_AREA_BLOCKED)
    damage = basechance * ( 1.0 - MentalPassiveD )
    ch_data = GetCharacterData(char)
    for ch_attr in ch_data['characterAttributes'][:ch_data['activeAttributes']]:
        if ch_attr == 'mindless':
            damage = 0
            break
        if ch_attr in ['fearless','dispassionate']:
            damage = damage / 2
        if ch_attr in ['conduit','glassjaw']:
            damage = damage * 2
        if ch_attr == 'weak minded':
            damage = damage * 1.5
        if ch_attr in ['timid','hot tempered']:
            damage = damage * 1.25
        if ch_attr in ['cybernetic brain','level headed','disciplined','tough guy']:
            damage = damage / 1.5
    MentalDamageData[char] = damage*0.01
    return damage*0.01

