# convertffxcustomdata.py
# (c) stumpy, 2007

# See if we can parse customs files and import one to another.
#
# The idea here is that we can read a ffxcustoms.py file from an older
# version of ffx and translate it into a new one, where we have also
# read an example new one to compare. This assumes that the new one
# has more arrays in it than the old one does.
#
# We read the type settings for each array with the same name and use
# that to navigate translations between changed positions within the
# array (where the same field is in a different position). We also
# take from the new one its default settings to add to any missing
# from the old one.
#
# NOTES:
#
# - Only uses last of duplicated custom arrays
# - Doesn't get rid of duplicate character entries in an array
# - Assumes first instance of multiple 'default' entries is the one to
# use in setting new fields
# - If USENEWCHAR is set, replaces character customization entries for
# characters in old array with ones from new array. Otherwise, leaves
# old entries alone

#Update 2007-01-27: changed default source and output to ffxcustom.py.old and ffxcustom.py.new - Ep.

import types
import os.path
import sys
import string

# Choosing which values to keep:
# - If character is only in the old array, keep it, adding any new
#   field data from the new array's defaults.
# - If character is only in the new array, add it.
# - If character is in both arrays, decide based on USENEWCHAR
#   setting:
#   - USENEWCHAR = 0    Use values from old array
#   - USENEWCHAR = 1    Use values from new array
#   - USENEWCHAR = 2  Check each field and if value in new array's
#     field is not blank, use it, otherwise keep old value
#   - USENEWCHAR = 3  Check each field and if value in old array's
#     field is not blank, keep it, otherwise use new value
USENEWCHAR = 3

# The process here will be to turn the old array set into the new
# ones, by changing their structure (combining, etc), adding in the
# new arrays to the old set, then adding characters from the arrays in
# the new set to the old one.
def ConvertCustomizations(f_old='',f_new='',f_conv='ffxcustom.py'):
    # get the data from each customization file
    try:
        f_old = os.path.abspath(GetOldFilename(f_old))
        cold = {}
        execfile(f_old,globals(),cold)
    except:
        print "Error: couldn't read old customization data from file %s" % repr(f_old)
        raise
    try:
        f_new = os.path.abspath(GetNewFilename(f_new))
        cnew = {}
        execfile(f_new,globals(),cnew)
    except:
        print "Error: couldn't read new customization data from file %s" % repr(f_new)
        raise
    # in case it's important, let's grab the order of the arrays
    f = open(f_new,'r')
    lines = f.readlines()
    f.close()
    OrderedVars = []
    for line in lines:
        if line[-3:]=='=1\n' or line[-3:]=='=0\n' or line[-3:]=='=[\n':
            name = line[:-3]
            if name not in OrderedVars:
                OrderedVars.append(name)
    # go through each variable (scalar or array) in new and find matches in old
    for nvar in OrderedVars:
        if nvar in cold.keys(): # the old custom set had this array
            ##print '%s (in old array)' % repr(nvar)
            # the FFX game options (like whether frozen characters take damage)
            if isinstance(cnew[nvar],types.IntType):
                if not cold.has_key(nvar):  # keep old options if they exist, else add new one
                    cold[nvar] = cnew[nvar]
            # Now do each of the customization arrays
            else:
                # check for the arrays that have changed or been combined
                if nvar == 'FFX_ABSORPTIONA_CUSTOM' and cold.has_key('FFX_ABSORPTIONA2_CUSTOM'):
                    # combine the old array
                    for linenum in range(len(cold[nvar])):
                        char = cold[nvar][linenum][0]
                        cold[nvar][linenum] = cold[nvar][linenum] + \
                                              getLineByTemplate(char,cold['FFX_ABSORPTIONA2_CUSTOM'])[1:]
                elif nvar == 'FFX_ILLUSIONIST_CUSTOM' and cold.has_key('FFX_ILLUSIONIST2_CUSTOM'):
                    # combine the old array
                    for linenum in range(len(cold[nvar])):
                        char = cold[nvar][linenum][0]
                        cold[nvar][linenum] = cold[nvar][linenum] + \
                                              getLineByTemplate(char,cold['FFX_ILLUSIONIST2_CUSTOM'])[1:]
                elif nvar == 'FFX_SCAN_CUSTOM' and cold.has_key('FFX_SCANFX_CUSTOM'):
                    # combine the old array
                    for linenum in range(len(cold[nvar])):
                        char = cold[nvar][linenum][0]
                        cold[nvar][linenum] = cold[nvar][linenum] + \
                                              getLineByTemplate(char,cold['FFX_SCANFX_CUSTOM'])[1:]
                elif nvar == 'FFX_PLASMA_CUSTOM' and cold.has_key('FFX_PLASMACMD_CUSTOM'):
                    # combine the old array
                    for linenum in range(len(cold[nvar])):
                        char = cold[nvar][linenum][0]
                        cold[nvar][linenum] = cold[nvar][linenum] + \
                                              getLineByTemplate(char,cold['FFX_PLASMACMD_CUSTOM'])[1:]
                elif nvar == 'FFX_MAGNETIC_CUSTOM':
                    # add the new field, taking the defaults and char entry from the new array
                    for linenum in range(len(cold[nvar])):
                        char = cold[nvar][linenum][0]
                        cold[nvar][linenum].append(getByTemplate(char,cnew[nvar],5))
                # Now, add any characters that are in the new array but
                # not in the old (presumably given as examples)
                oldchars = map(lambda line: line[0], cold[nvar])
                for line in cnew[nvar]:
                    if line[0] not in oldchars:
                        ##print '  adding char %s to old array' % repr(line[0])
                        cold[nvar].append(line)
                    elif USENEWCHAR==1:
                        ##print '  replacing %s in old array' % repr(line[0])
                        charindex = oldchars.index(line[0])
                        cold[nvar][charindex] = line
                    elif USENEWCHAR==2:
                        ##print '  changing %s in old array, preferring non-blank new' % repr(line[0])
                        charindex = oldchars.index(line[0])
                        for field in range(len(cold[nvar][charindex])):
                            if line[field]: # entry in new array is not blank
                                cold[nvar][charindex][field] = line[field]
                    elif USENEWCHAR==3:
                        ##print '  changing %s in old array, preferring non-blank old' % repr(line[0])
                        charindex = oldchars.index(line[0])
                        for field in range(len(cold[nvar][charindex])):
                            if not cold[nvar][charindex][field]: # entry in old array is blank
                                cold[nvar][charindex][field] = line[field]
        else:   # this array is new to the new version
            ##print '%s (not in old array)' % repr(nvar)
            cold[nvar] = cnew[nvar]
    # Should we try to alter the custom arrays at the bottom of ffx.py? I am going to skip that
    # Now rename the old customization file as the backup
    f_conv = GetConvertedFilename(f_conv)
    if os.path.isfile(f_conv):  # the new file already exists
        try:    # create a backup
            i = 1
            BackupFileName = '%s.bak' % (f_conv,)
            while os.path.isfile(BackupFileName):
                i = i + 1   # first numbered .bak will be '.bak2'
                BackupFileName = '%s.bak%d' % (f_conv,i)
            os.rename(f_conv,BackupFileName)
        except OSError:
            print 'ConvertCustomizations: Read-Only problem backing up <%s>.  Quitting' % (f_conv,)
            raise
        except:
            print 'ConvertCustomizations: Unable to create backup for <%s>.  Quitting' % (f_conv,)
            raise
    # now write the converted array
    f = open(f_conv,'w')
    for nvar in OrderedVars:
        if isinstance(cold[nvar],types.IntType):
            f.write('%s=%d\n' % (nvar,cold[nvar]))
        else:
            f.write('\n%s=[\n' % nvar)
            for line in cold[nvar]:
                f.write('%s,\n' % GetListString(line))
            f.write(']\n')
    f.close()


# Silly, but I want to preserve the double-quote preference we have
# been using for strings D1list is a 1 deep list, e.g.
# ["default","effect_ffx_energysparks","ffx_stockpile"]
def GetListString(D1list):
    s = '['
    for item in D1list:
        if isinstance(item,types.StringType):
            s = s+'"' + item + '",'
        else:
            s = s+('%s'%item)+','
    s = s[:-1] + ']'
    return s

# cribbed from ffx.py                    
def getByTemplate(char,array,index):
    result=array[0][index]
    for elem in array:
        if (elem[0]==char):
            result=elem[index]
            break
    return result

def getLineByTemplate(char,array):
    result=array[0]
    for elem in array:
        if (elem[0]==char):
            result=elem
            break
    return result

# Do some file name guessing. These are for IN-GAME use (from a script
# or from the console).

# return the name of the old customizations file, defaulting to
# ffxcustom.py.old in the mod's missions/scripts folder
def GetOldFilename(fname):
    if fname == '':
        fname = 'ffxcustom.py.old'
    if os.path.dirname(fname) == '':
        fname = os.path.join(GetModPath(),'missions','scripts',fname)
    return fname

# return the name of the new customizations file, defaulting to
# ffxcustom.py.new in the mod's missions/scripts folder
def GetNewFilename(fname):
    if fname == '':
        fname = 'ffxcustom.py.new'
    if os.path.dirname(fname) == '':
        fname = os.path.join(GetModPath(),'missions','scripts',fname)
    return fname

# return the name of the old customizations file, defaulting to
# ffxcustom.py.old in the mod's missions/scripts folder
def GetConvertedFilename(fname):
    if fname == '':
        fname = 'ffxcustom.py'
    if os.path.dirname(fname) == '':
        fname = os.path.join(GetModPath(),'missions','scripts',fname)
    return fname

# taken from datfiles.py

# Determine which version of FF is running.  Returns default if not run from within the
# game.  Otherise returns 1 for Freedom Force and 2 for Freedom Force vs. The Third Reich.
def ffver(default = 2):
    if sys.argv[0] == 'FreedomForce':
        return 1
    elif sys.argv[0] == 'Freedom Force vs the 3rd Reich':
        return 2
    else:
        print 'ffver: unrecognized game version, returning %s' % str(default)
        return default

# Find the path where the basic mod DATs (e.g. characters, powers,
# objects, etc.) are when run during a mission.  This path is returned
# lower-case. Added some I/O to get this working with FFvtTR and added
# caching so that frequent calls aren't impeded by the I/O.  Update:
# I/O not needed, keeping caching anyway.
def GetModPath(ForceCheck=0):
    global _ModPath
    if ( not ForceCheck ) and ('_ModPath' in globals().keys() ):
        return _ModPath
    _ModPath = ''
    syspath = map(string.lower,sys.path)
    workingdir = string.lower(os.getcwd())
    if ffver() == 1:     # for v1 we can get this from the system path.
        tailstring = 'temp'
    elif ffver() == 2:
        tailstring = 'missions\\scripts'
    else:   # likely not in game
        return ''
    for p in syspath:
        if ( p[:len(workingdir)]==workingdir ) and ( p[-len(tailstring):]==tailstring ):
            _ModPath = os.path.normpath(p[:-len(tailstring)])
    return _ModPath