######################################################################
## 
## Fire Hydrant Improvement mission plugin
## By pimthe
##
######################################################################

# DESCRIPTION
# This plugin adds new behaviours to fire hydrants: When broken, a jet of water is released, causing a bit of damage
# and knockback. In addition, nearby flame walls and fire cages are extinguished and fire-based characters get
# power nullified, while water-based/frozen-material characters get energised and receive a defence against fire.

# DEPENDENCIES
# - Plugins Importer (obviously) included with FFX 3.2+
# - Merged powers.dat (done by default in FFX 3.2+) for ffq_firehydrant power and ffq_firehydrant_shield defense
# - Generalutils.py (included with FFX 3.2+)

# INCOMPATIBILITIES
# - None known.

# CUSTOMISATION
# The list of flame objects and of non fire/frozen-material characters receiving beneficial or detrimental effects
# is just below.

# VERSION HISTORY
# 1.0, 2006-07-15:
# - First release.



################ CUSTOMISATION: ADD YOUR TEMPLATES HERE ################
# templates of objects which should act as fire hydrants
fireHydrant_Templates = ("fire_hydrant", "pat_fire_hydrant",)
# templates of FFX objects which can be extinguished        
flameObjects = ("ffx_flames", "ffx_firecage",)
# templates of characters whose material is not fire but whose powers (or at least those which consume energy) won't work when soaked
fireTemplates = ("mech_man_flame", "mech_man_flame_extra", "pyro", "pyroness", "red_sun", "red_sun_extra",)
# templates of characters whose material is not ice but whose powers benefit from it
waterTemplates = ("anemone", "aqualad", "aquaman", "atlantean", "attuma", "hydroman", "krang", "orka", "man_o_war_extra", "man_o_war_n", "man_o_war", "namor", "namorita", "sea_urchin", "sea_urchin_extra", "sea_urchin_n", "sub-mariner",)
# Note on fireTemplates and waterTemplates: No need to write all the variants of a hero's name, as case and non alphanumerical characters
# are ignored. (i.e.,"Sub-Mariner", "sub_mariner" and "submariner!" would need only a single entry... but of course, you'd still need
# to add another one for "namor".)



import js
import cshelper
import ffx
import generalutils

# pseudo OnPostInit(), called by the plugin code after dynamic objects have started loading
def OnPostInit():
    #list of fire hydrants available on map. Note that this list is NOT updated during the mission to reduce performance hit.
    fireHydrant_Objects = []
    for obj in js.Mission_GetObjects():
        for fh in fireHydrant_Templates:
            if js.Object_GetTemplate(obj) == fh:
                fireHydrant_Objects.append(obj)
                break
    #register damage for existing fire hydrants
    for obj in fireHydrant_Objects:
        cshelper.regDamage(obj, 'OnFireHydrantDamaged')
        js.Object_SetAttr( obj, 'fh_pos_x', int( js.Get_ObjectPos(obj)[0] ) )
        js.Object_SetAttr( obj, 'fh_pos_y', int( js.Get_ObjectPos(obj)[1] ) )
        js.Object_SetAttr( obj, 'fh_pos_z', int( js.Get_ObjectPos(obj)[2] ) )


def OnFireHydrantDamaged(event):
    fh = event.object
    if js.Object_GetAttr(fh, 'health') > 7: #fire hydrants break at 7 HP
        cshelper.regDamage(fh, 'OnFireHydrantDamaged')
        return
    fhPos = ( js.Object_GetAttr(fh, 'fh_pos_x'), js.Object_GetAttr(fh, 'fh_pos_y'), js.Object_GetAttr(fh, 'fh_pos_z') )
    fhMarker = "%s_fh" %fh #not a real marker, because markers can't trigger explosions
    js.Object_SpawnAt('ffx_target', fhPos, fhMarker)
    js.Trigger_Explosion(fhMarker, 'ffq_firehydrant')
    js.Object_Destroy('fhMarker')
    #find objects near exploding fire hydrant
    objects = js.Mission_GetObjects()
    nearObjects = []
    for o in objects:
        if ffx.distanceSq(fhMarker,o) < 4000:
            nearObjects.append(o)
    #find whether near objects are potential targets of negative or positive effects
    for target in nearObjects:
        template = ffx.FFX_GetTemplate(target)
        if (js.Object_GetClass(target) & js.OC_VEHICLE) or (js.Object_GetClass(target) & ffx.OC_GENERIC):
            #is the target an FFX flame object which can be extinguished?
            for entry in flameObjects:
                if template == entry:
                    js.Object_Destroy(target)
                    break
        elif js.Object_GetClass(target) & ffx.FFX_CHARACTER:
            #is the target a character of the fire material or of a fire-centric template?
            fire = 0
            if js.Object_GetAttr(target, 'material') == 4: #4 is fire material
                fire = 1
            else:
                for entry in fireTemplates:
                    entry = generalutils.cleanUpString(entry)
                    template = generalutils.cleanUpString(template)
                    if template == entry:
                        fire = 1
                        break
            if fire:
                ffx.fallOver(target)
                js.Object_SetSecondaryState(target, js.SCSTATE_POWER_NULLIFICATION, 15, 0)
            #is the target a character of the ice material or of a water-centric template?
            else:
                water = 0
                if js.Object_GetAttr(target, 'material') == 9: #9 is ice material
                    water = 1
                else:
                    for entry in waterTemplates:
                        entry = generalutils.cleanUpString(entry)
                        template = generalutils.cleanUpString(template)
                        if template == entry:
                            water = 1
                            break
                if water:
                    js.Object_SetSecondaryState(target, js.SCSTATE_ENERGIZED, 15, 0)
                    js.Object_SetPrimaryState(target, js.PCSTATE_STUNNED, 0, js.REMOVE_STATE)
                js.Trigger_Power(target, target, 'ffq_firehydrant_shield', '')

