######################################################################
## 
## Zombie Improvement mission plugin
## By pimthe
##
######################################################################

# DESCRIPTION
# - This plugin increases the duration of zombies relative to the magnitude and level of the
#    power. Duration varies from 15 (1 * 5 + 0 * 5 + 10) to 65 (5 * 5 + 6 * 5 + 10) secs instead
#   of the usual ~12 secs.
# - Adds Degenerative attribute to each zombie to compensate for the increased duration, while 
#   giving some reason to attack a zombie rather than staying out of the way until he expires.
# - Zombies from the same side (i.e. serving a hero/police master vs. a villain/minion one)
#   usually won't attack each other anymore. Keep in mind that they won't still behave with
#   much intelligence. But hey, when was the last time you've met a bright zombie, anyway?

# Hopefully, this will make zombies more enjoyable to use. :)

# TO DO:
# - Characters with mental/mystical resistances/passive defences may resist being raised by an enemy.
# - Reduce duration of zombified clones
# - BUG: some zombified characters (heroes and maybe even custom villains using hero templates) won't attack

#NOTES/BUGS:
# - Multiple teams aren't recognized, as the zombie state AI behaviour is hard-coded.
# - Corpses of KO'ed zombies raised by heroes or allies disappear suddenly, without any fading.

# DEPENDENCIES
# - Plugins Importer (obviously) included with FFX 3.2+
# - Datfiles and Chardata modules (included with FFX 3.0+)

# INCOMPATIBILITIES
# - None known.

# CUSTOMISATION
# - None.

# VERSION HISTORY
# 0.9, 2006-07-23:
# - First release.
# 0.10, 2006-08-22:
# - Finally found a way to increase corpse duration. Added customisation for it.
# - Updated attribute addition / removal to new method addFFXAttr(()/removeFFXAttr().
# 0.11, 2006-08-29:
# - Removed the corpse persistence increase and customisation, since it stalled the game due the way the game
#   evaluates dead and alive status... :(
# 0.12, 2007-02-14:
# - Corrected names of addFFXAttrib()/removeFFXAttrib() (thanks to TaskMasterX!)




import js
import ff
import cshelper
import ffx
import m25ai
import datfiles
import chardata


# pseudo OnPostInit(), called by the plugin code after dynamic objects have started loading
def OnPostInit():
    for char in ffx.getAllCharacters(): #register existing characters
        ffx.RegDeath(char, 'OnZombieRegisterDeath', persistent = 1)
    ffx.regCharacterSpawned('OnZombieRegisterChar', persistent = 1) #register future characters

def OnZombieRegisterChar(event):
    char = event.object
    ffx.RegDeath(char, 'OnZombieRegisterDeath', persistent = 1)

def OnZombieRegisterDeath(event):
    char = event.object
    if not js.Object_Exists(char):
        return
    #destroy KO'ed allied minion zombies,as their bodies don't disappear because of AI_MakeHeroAI()
    #using AI_DestroyHeroAI() doesn't make them fade,it simply removes the 'flyover' 
    #note that AI_MakeHeroAI() sets the 'noFade' object attribute, but the value itself is ignored!
    if js.Object_GetAttr(char, 'zombieState'):
        if ffx.FFX_ObjectGetAttr(char, 'ffq_zombie_degenSet'):
            ffx.removeFFXAttrib(char, 'degenerative')
        if ffx.FFX_ObjectGetAttr(char, 'ffq_zombie_heroai'):
            js.AI_DestroyHeroAI(char)
            ffx.RegTimer('OnZombieRemoveCorpse', 15, 0, char)
    else:
        ffx.regSecondaryState2(char, js.SCSTATE_ZOMBIE, 'OnZombieRegisterZombie')

def OnZombieRemoveCorpse(event):
    char = event.object
    if not js.Object_Exists(char):
        return
    if js.Object_IsAlive(char):
        return
    js.Object_Destroy(char)

def OnZombieRegisterZombie(event):
    zombie = event.object
    #regDeath() is called twice in a row for some unknown reason :(
    #however, this check can't be done in OnZombieRegisterDeath() because of possible resurrections
    alreadyCalled = ffx.FFX_ObjectGetAttr(zombie, 'ffq_zombie_regdeath')
    if alreadyCalled > 0:
        return
    else:
        ffx.FFX_ObjectSetAttr(zombie, 'ffq_zombie_regdeath', 1)
    zombieMasters = getAllZombieMasters()
    if len(zombieMasters) == 0:
        return
    myMaster = ffx.sortByDistance(zombie, zombieMasters)[0] #this presumes that the nearest character with zombie powers is the summoner (if not, well, let's assume that he gains control of him because he's nearer
    duration = (ffx.FFX_ObjectGetAttr(myMaster, 'ffq_zombie_powerlevel') * 5) + (ffx.FFX_ObjectGetAttr(myMaster, 'ffq_zombie_powermagnit') * 5) + 10
    js.Object_SetSecondaryState(zombie, js.SCSTATE_ZOMBIE, duration)
    if not ffx.hasAttribute(zombie, 'degenerative'):
        ffx.addFFXAttrib(zombie, 'degenerative')
        ffx.FFX_ObjectSetAttr(zombie, 'ffq_zombie_degenSet', 1)
    #solves the problem of zombie state completely overriding any order to the existing AI
    #note that this doesn't support multiple teams, however
    if ffx.isEvil(myMaster):
        js.AI_DestroyHeroAI(zombie)
    else:
        #detect that the body won't disappear as it should due to the added hero AI
        if not js.Object_GetClass(zombie) & js.OC_CONTROLLABLE:
            ffx.FFX_ObjectSetAttr(zombie, 'ffq_zombie_heroai', 1) 
        js.AI_MakeHeroAI(zombie)
    myMasterTeam = m25ai.GetTeam(myMaster)
    if myMasterTeam == 0:
        m25ai.SetDefaultTeam(myMaster)
        myMasterTeam = m25ai.GetTeam(myMaster)
    if myMasterTeam != m25ai.GetTeam(zombie):
        m25ai.SetTeam(zombie, myMasterTeam)
    #makes civilian zombies react
    targets = ffx.getAllAdversaries(myMaster, lures = 1)
    cshelper.addKillGoal(zombie, tuple(targets))
    #forces the zombie to go - hopefully - near enough an enemy to activate the AI reactions
    js.Trigger_Move(zombie, 'move', js.Get_ObjectPos( ffx.getNearestAdversary(zombie) ) )
    

def getAllZombieMasters():
    zombieMasters = []
    templatesChecked = []
    for char in ffx.getAllCharacters():
        if ffx.FFX_ObjectHasAttr(char, 'ffq_zombie_powerlevel'):
            if ffx.FFX_ObjectGetAttr(char, 'ffq_zombie_powerlevel') > 0:
                zombieMasters.append(char)
        else:
            powerLevel = 0
            template = ffx.FFX_GetTemplate(char)
            if not template in templatesChecked:
                templatesChecked.append(template)
                zombiePowers = []
                for entry in chardata.Character_GetSpecialDamageTypePowers(char, datfiles.PT_SPECIAL_ANIMATE_DEAD, ignoreBought = ff.DEBUG_ALLPOWERS): #DEBUG_ALLPOWERS works only on characters created after initialisation, but it's the best I can find
                    zombiePowers.append(entry['PowerName'])
                for entry in chardata.Character_GetSpecialDamageTypePowers(char, datfiles.PT_SPECIAL_ANIMATE_DEAD_MINIONS, ignoreBought = ff.DEBUG_ALLPOWERS):
                    zombiePowers.append(entry['PowerName'])
                if len(zombiePowers) > 0:
                    for thisPower in zombiePowers: # obviously this is a bit overkill, as it would be surprising if a character had more than one zombie power...
                        powerLevel = max(powerLevel, chardata.GetCharacterData(char)['powerLevels'][thisPower])
                ffx.FFX_ObjectSetAttr(char, 'ffq_zombie_powerlevel', powerLevel)
                if powerLevel > 0:
                    zombieMasters.append(char)
                # determine the actual magnitude of the powers
                # TO DO: parse again if powers.dat has been rewritten since last time
                for thisPower in zombiePowers:
                    powerMagnitude = datfiles.Campaign_ReadPowers()[thisPower]['Magnitude'] # a number from 0-6
                    ffx.FFX_ObjectSetAttr(char, 'ffq_zombie_powermagnit', powerMagnitude)
    return zombieMasters

