# readsavedevents.py, by stumpy
#
# Read through a Freedom Force saved game file and return a list of
# all the event sinks in the game engine.  Potentially useful for
# debugging and optimization.  Note that this can be run in-game or in
# a separate python command window. WHen run in-game, it may be useful
# to combine with the datfiles filename-guessing utility functions,
# like GetMostRecentSavedGameFile().  I have not optimized this code
# at all; it is not intended for real-time use.
#
# Example:  (in-game)
#   from datfiles import *
#   from readsavedevents import *
#   events = ReadSavedEvents(GetMostRecentSavedGameFile(),verbose=1)

print "loading readsavedevents 0.105"

import string
import struct
import types

# String names of the event types, for readability
EVENT_NAMES = \
           {0 : 'EVENT_OBJ_PROXIMITY_ENTER',
            1 : 'EVENT_OBJ_PROXIMITY_LEAVE',
            2 : 'EVENT_OBJ_HEALTHCHANGED',
            3 : 'EVENT_OBJ_SHIELD_DAMAGED',
            4 : 'EVENT_MARKER_LEAVE',
            5 : 'EVENT_MARKER_ENTER',
            12 : 'EVENT_ANIMATION_DONE',
            13 : 'EVENT_AI_PATROL',
            14 : 'EVENT_AI_COLLECT',
            15 : 'EVENT_AI_DANGER',
            17 : 'EVENT_TIMER',
            19 : 'EVENT_DETECT_ENEMY',
            20 : 'EVENT_TEMPLATE_OBJ_DIED',
            21 : 'EVENT_POWER_HIT',
            23 : 'EVENT_BLD_SEGMENT_COLLAPSED',
            25 : 'EVENT_CHARACTER_STATE_CHANGED',
            28 : 'EVENT_PICKUP',
            29 : 'EVENT_THROW',
            30 : 'EVENT_DLG_ONSHOW',
            31 : 'EVENT_DLG_ONHIDE',
            32 : 'EVENT_MISSIONLOST',
            33 : 'EVENT_CHARACTER_SEC_STATE_CHANGED'}

# Grab the event sinks stored in the saved game file filename.  These
# are returned as a list of events, with each event as a tuple
# containing the event type, callback function, event.string,
# event.object, event.userdata, event.float, and event.persist. Call
# with verbose==1 to get a crude printout.
def ReadSavedEvents(filename, verbose=0):
    '''Parse a saved game (given as a full path in filename) and grab the registered game engine and proximity sinks.  verbose==1 prints them as they are found.'''
    f=open(filename,'rb')
    dat=f.read()
    f.close()
    events = []
    hpos = string.find(dat,'HDR4\x0C\x00\x00\x00CEngineEvent')
    if verbose:
        print '%2s\t%20s\t%12s\t%12s\t%12s\t%5s\t%5s\t%5s' % \
              ('Code','Event_Name','Callback Fn','string','object','user','float','persist')
    while ( hpos > -1 ):
        hposnext = string.find(dat,'HDR4\x0C\x00\x00\x00CEngineEvent',hpos+1)
        if hposnext > -1:
            block = dat[hpos:hposnext]  # get up to the next event block
        else:
            block = dat[hpos:hpos+1024] # get the last one
        eventType = ord(block[29])
        funclen = ord(block[33])
        bpos = 37   # start of function string
        eventFunction = block[bpos:bpos+funclen]
        objlen = ord(block[bpos+funclen])
        bpos = bpos + funclen + 4
        eventObject = block[bpos:bpos+objlen]
        strlen = ord(block[bpos+objlen])
        bpos = bpos + objlen + 4
        eventString = block[bpos:bpos+strlen]
        bpos = bpos + strlen
        (eventFloat,eventUser,eventPersist) = struct.unpack('fff',block[bpos:bpos+12])
        events.append((eventType,eventFunction,eventString,eventObject,eventUser,eventFloat,eventPersist))
        if verbose:
            eventName = EVENT_NAMES.get(eventType)
            print '%2d\t%20s\t%12s\t%12s\t%12s\t%2.2f\t%2.2f\t%2.2f' % \
                   (eventType,eventName[6:],eventFunction,eventString,eventObject,eventUser,eventFloat,eventPersist)
        hpos = hposnext
    # Marker proximity events are done differently
    hpos = string.find(dat,'HDR4\x15\x00\x00\x00CMarkerProximityEvent')
    while ( hpos > -1 ):
        hposnext = string.find(dat,'HDR4\x15\x00\x00\x00CMarkerProximityEvent',hpos+1)
        if hposnext > -1:
            block = dat[hpos:hposnext]  # get up to the next event block
        else:
            block = dat[hpos:hpos+1024] # get the last one
        eventType = ord(block[38])
        funclen = ord(block[42])
        bpos = 46   # start of function string
        eventFunction = block[bpos:bpos+funclen]
        objlen = ord(block[bpos+funclen])
        bpos = bpos + funclen + 4
        eventObject = block[bpos:bpos+objlen]
        strlen = ord(block[bpos+objlen])
        bpos = bpos + objlen + 4
        eventString = block[bpos:bpos+strlen]
        bpos = bpos + strlen
        (eventFloat,eventUser,eventPersist) = struct.unpack('fff',block[bpos:bpos+12])
        bpos = bpos + 12
        postuple = struct.unpack('fff',block[bpos:bpos+12])
        events.append((eventType,eventFunction,eventString,eventObject,eventUser,eventFloat,eventPersist,postuple))
        if verbose:
            eventName = EVENT_NAMES.get(eventType)
            print '%2d\t%20s\t%12s\t%12s\t%12s\t%2.2f\t%2.2f\t%2.2f\t%s' % \
                   (eventType,eventName[6:],eventFunction,eventString,eventObject,eventUser,eventFloat,eventPersist,postuple)
        hpos = hposnext
    return events

# print out the event list in a somewhat readable form
def ShowEventList(elist):
    '''print out the event list in a somewhat readable form'''
    # find decent lengths for printing each field
    functionlen = max(map(lambda e:len(e[1]),elist))
    stringlen = max(map(lambda e:len(e[2]),elist))
    objectlen = max(map(lambda e:len(e[3]),elist))
    userlen = max(map(lambda e:len('%2.2f'%e[4]),elist))
    floatlen = max(map(lambda e:len('%2.2f'%e[5]),elist))
    persistlen = max(map(lambda e:len('%2.2f'%e[6]),elist))
    formatstr = '%4d %29s %' + `functionlen` + 's %' + `stringlen` + 's %' + `objectlen` + 's %' + `userlen` + '.2f %' + `floatlen` + '.2f %' + `persistlen` + '.2f'
    formathdr = '%4s %29s %' + `functionlen` + 's %' + `stringlen` + 's %' + `objectlen` + 's %' + `userlen` + 's %' + `floatlen` + 's %' + `persistlen` + 's'
    ##print '<%s>' % formathdr
    print formathdr % ('code','Event_Name','Callback Fn','string','object','user','float','persist')
    print '-'*len(formathdr % ('code','Event_Name','Callback Fn','string','object','user','float','persist'))
    for e in elist:
        print formatstr % (e[0],EVENT_NAMES.get(e[0])[6:],e[1],e[2],e[3],e[4],e[5],e[6])

def GetEventTypes(EventList,EventType):
    '''Return a list of all events in EventList with the same EventType.  Useful for finding all timer events, etc.'''
    return filter(lambda e: e[0]==EventType,EventList)

# Given a list, return a the items in the list that are duplicated.
# What is returned is a dictionary with the duplicated list items as
# keys and the number of occurances of that item in the original list
# as the values.  NB: The obvious assumption here is that the items in
# the list are suitable as dictionary mapping keys.  That includes
# most, but not all, objects.  If ignorePersist==1 (the default), then
# if the items in the list are sequence types, the last item in each
# sequence is ignored (which is the persist field - often filled with
# semi-arbitary values by the engine).
def FindDuplicates(list,ignorePersist=1):
    '''Look at the saved event list and make a dictionary of duplicated SE entries.  ignorePersist is useful for grouping timer events, since the "same" entry will have different persist values depending on the internal game timer.'''
    dups = {}
    if ignorePersist and isinstance(list[0],(types.ListType,types.TupleType)):
        list = map(lambda i: i[:-1],list)
    for i in range(len(list)):
        item = list[i]
        if item in list[i+1:]:
            if item in dups.keys():
                dups[item] = dups[item]+1
            else:
                dups[item] = 2
    return dups

##def FindDuplicates(list,ignorePersist=1):
##    dups = {}
##    list.sort()
##    if ignorePersist and isinstance(list[0],(types.ListType,types.TupleType)):
##        list = map(lambda i: i[:-1],list)
##    i = 0
##    listlen = len(list)
##    while ( i < listlen ):
##        event = list[i]
##        count = 1
##        while ( i+count < listlen ) and ( event == list[i+count] ):
##            count = count + 1
##        dups[event] = count
##        i = i + count
##    return dups

# Look at the SE list and make a dictionary of how many times each SE
# is registered.  Very similar to FindDuplicates, but also includes
# SEs that are registered only once.
def FindGroups(list,ignorePersist=1):
    '''Look at the saved event list and make a dictionary of how many times each SE is registered.'''
    groups = {}
    list.sort()
    if ignorePersist and isinstance(list[0],(types.ListType,types.TupleType)):
        ##list = map(lambda i: i[:-1],list)
        i = 0
        listlen = len(list)
        while ( i < listlen ):
            event = list[i][:-1]
            count = 1
            while ( i+count < listlen ) and ( event == list[i+count][:-1] ):
                count = count + 1
            groups[event] = count
            i = i + count
        return groups
    else:
        i = 0
        listlen = len(list)
        while ( i < listlen ):
            event = list[i]
            count = 1
            while ( i+count < listlen ) and ( event == list[i+count] ):
                count = count + 1
            groups[event] = count
            i = i + count
        return groups

def ShowEventDict(edict):
    '''Print out the dictionary of grouped events generated by functions like FindGroups and FindDuplicates.'''
    ekeys = edict.keys()
    ekeys.sort()
    for k in ekeys:
        print '%6d\t%s\t%s' % (edict[k],EVENT_NAMES.get(k[0]),k)

def SummarizeByEventType(elist):
    '''show the number of events in the list of each event type'''
    d = {}
    etypes = EVENT_NAMES.keys()
    etypes.sort()
    print '%33s: %s' % ('Event Type','Count')
    for etype in etypes:
        d[etype] = filter(lambda e: e[0]==etype,elist)
        count = len(d[etype])
        print '%33s: %d' % (EVENT_NAMES[etype],count)
    return d

def SummarizeByCallback(elist,CBMatchStr='',verbose=1):
    '''get all the events in the list that have CBMatchStr as part of their callback function'''
    CBs = {}
    maxlen = 0
    for e in elist:
        if not CBMatchStr:
            if e[1] in CBs.keys():
                CBs[e[1]].append(e)
            else:
                CBs[e[1]] = [e]
                if len(e[1]) > maxlen:
                    maxlen = len(e[1])
        else:
            if e[1].find(CBMatchStr)>=0:
                if e[1] in CBs.keys():
                    CBs[e[1]].append(e)
                else:
                    CBs[e[1]] = [e]
                    if len(e[1]) > maxlen:
                        maxlen = len(e[1])
    if verbose:
        CBkeys = CBs.keys()
        CBkeys.sort()
        printstr = '%'+('%d'%maxlen)+'s:%5d'
        for CB in CBkeys:
            print printstr % (CB,len(CBs[CB]))
    return CBs
    